from fast_di.depends.depends import FastDI
from fast_di.depends.depends_stub import FastDIStub
from fast_di.init_dependencies import init_dependencies
from fast_di.collector.collector import DependencyCollector

__all__ = ["Stub", "FastDI", "FastDIStub", "DependencyCollector", "init_dependencies"]

__version__ = "0.0.2"
__author__ = "Abrekov Alim"
